package be.iminds.ilabt.jfed.rspec.model;


import be.iminds.ilabt.jfed.lowlevel.TestbedInfoSource;
import be.iminds.ilabt.jfed.rspec.parser.ExtraXml;
import be.iminds.ilabt.jfed.util.GeniUrn;

import javax.annotation.Nonnull;
import javax.annotation.Nullable;
import java.util.List;

/**
 * RspecLink
 */
public interface RspecLink {
    /**
     * replace all properties with the properties of another link
     * this assumes the relevant id's are the same.
     *
     * Useful when merging manifests.
     *
     * @param otherLink the other link, from which the properties have to be used
     * @param overwriteWithNulloverwriteNonAuthoratativeComponentManagers if true, set properties from this link to null, if they are null at otherLink and remove from list if not in otherLink's list.
     *                          If false, keep properties from this link unchanged, if they are null at otherLink and only add to lists.
     */
    void overwritePropertiesWith(@Nonnull RspecLink otherLink,
                                 boolean overwriteWithNulloverwriteNonAuthoratativeComponentManagers);

    @Nonnull ModelRspec getRspec();

    @Nonnull String getUniqueId();
    @Nullable String getClientId();
    void setClientId(@Nullable String value);

    //read write!
    @Nonnull List<GeniUrn> getComponentManagerUrns();

    @Nonnull List<String> getLinkTypes();

    @Nonnull List<? extends LinkSetting> getLinkSettings();
    @Nullable LinkSetting getLinkSetting(@Nonnull RspecInterface fromIface, @Nonnull RspecInterface toIface);
    @Nullable LinkSetting getLinkSettingByClientIds(@Nonnull String fromIfaceName, @Nonnull String toIfaceName);
    @Nullable LinkSetting getLinkSettingByUniqueIds(@Nonnull String fromIfaceName, @Nonnull String toIfaceName);

    boolean isImpaired();

    /**
     * Interfaces are added to this list via RspecInterface. Deleting an interface is done by
     * {@link be.iminds.ilabt.jfed.rspec.model.javafx_impl.FXRspecInterface#delete()}
     *
     * @return
     */
    @Nonnull List<? extends RspecInterface> getInterfaces();
    @Nullable RspecInterface getInterfaceForNode(@Nonnull RspecNode node);
    @Nullable RspecInterface getInterfaceByUniqueId(@Nonnull String uniqueId);
    @Nullable RspecInterface getInterfaceByClientId(@Nonnull String clientId);

    @Nullable GeniUrn getSliverId();

    void setSliverId(@Nullable String sliverId);
    void setSliverId(@Nullable GeniUrn sliverId);

    /**
     * TestbedInfoSource is used to determine if two different URNs are actually the same authority.
     * This happens for exogeni for example. In that case, a link with multiple component managers can be a non-stitched link!
     *
     * if testbedInfoSource is null, it is assumed that any link (of type "lan") with multiple component managers is a stitched link
     *
     * note that only links of type "lan" can be stiched links. Links of type "egre" and "gre" are never stitched links.
     * */
    boolean isStitched(@Nullable TestbedInfoSource testbedInfoSource);

    @Nonnull List<ExtraXml> getExtraXml();

    //sharedlan extension
    /**
     * @return null if no shared lan, shared lan name otherwise
     */
    @Nullable String getSharedLan();
    void setSharedLan(@Nullable String sharedLanName);


    @Nullable Integer getVlanTag();
    void setVlanTag(@Nullable Integer vlanTag);

    boolean isVlanTaggingEnabled();
    boolean hasVlanTaggingTag();
    /** set to null to remove tag completely */
    void setVlanTagging(@Nullable Boolean value);

    boolean isNoMacLearning();
    boolean hasNoMacLearningTag();
    /** set to null to remove tag completely */
    void setNoMacLearning(@Nullable Boolean value);
    /* very "funny": can be "yep" instead of "true", and we should reconstruct it that way. */
    @Nullable String getNoMacLeaningValueString();
    void setNoMacLearning(@Nullable String value);

    boolean isBestEffortEnabled();

    boolean hasBestEffortTag();

    void setBestEffort(@Nullable Boolean value);

    boolean isLinkMultiplexingEnabled();

    boolean hasLinkMultiplexingTag();

    void setLinkMultiplexing(@Nullable Boolean value);

    boolean hasAutoShareLan();
    @Nonnull List<String> getAutoShareLanNames();

    boolean isInterswitchDisallowed();
    boolean hasInterswitchDisallowTag();
    void setInterswitchDisallow(@Nullable Boolean value);

}
